require 'active_model_serializers'
require 'jsonapi_ams_extensions/version'

module JsonapiAmsExtensions
  def self.included(klass)
    ActiveModelSerializers.config.key_transform = :underscore
    ActiveModel::Serializer.config.adapter = :json_api

    klass.extend ClassMethods
    # If we're using forked AMS
    # https://github.com/rails-api/active_model_serializers/pull/1720
    if klass.respond_to?(:associations_via_include_param)
      klass.associations_via_include_param(true)
    end
  end

  class ResourceType
    def self.for(serializer)
      klass      = ActiveModelSerializers::Adapter::JsonApi::ResourceIdentifier
      identifier = klass.new(serializer, {})
      identifier.as_json[:type].to_sym
    end
  end

  module ClassMethods
    def extra_attribute(name)
      attribute name, if: :"allow_#{name}?"

      define_method :"allow_#{name}?" do
        if extra_fields = instance_options[:extra_fields]
          if extra_fields = extra_fields[ResourceType.for(self)]
            extra_fields.include?(name)
          end
        end
      end
    end

    def extra_attributes(*names)
      names.each do |name|
        extra_attribute(name)
      end
    end

    def boolean_attribute(name, *args)
      is_method = :"is_#{name.to_s.sub('?', '')}"
      attribute(is_method, *args)

      define_method(is_method) do
        object.read_attribute_for_serialization(name)
      end
    end

    def boolean_attributes(*attributes)
      attributes.each do |attr|
        boolean_attribute(attr)
      end
    end
  end
end
